@@ -332,7 +332,7 @@ program sound_1d
      if(draw_flag(2:2)=='o')then
         call DclSetParm( 'GRAPH:LCLIP', .true. )
         call SkewT( trim(fname(2,i)), 20.0, (/240.0, 300.0/),  &
-  &                 draw_region_p, temp, pres, qv, undef=undef )
+  &                 draw_region_p, temp, pres, qv, undef=undef, refh=p_ref )
         if(draw_flag(3:3)=='o')then
            call DclSetParm( 'GRAPH:LCLIP', .false. )
            CALL SGQVPT( vx_new(1), vx_new(2), vy_new(1), vy_new(2) )
@@ -436,7 +436,7 @@ program sound_1d
      tmp_char='('
      t_c=''
 
-     tmp_char='(a20,'//trim( adjustl( i2c_convert( jmax ) ) )//'f)'
+     tmp_char='(a20,1P'//trim( adjustl( i2c_convert( jmax ) ) )//'E16.8)'
 write(*,*) "format", trim(tmp_char)
 
      do i=1,nnz
