@@ -94,7 +94,7 @@ write(*,*) "nl check", nl
   allocate(omeganr(0:nl/2-1,0:nl/2-1))
   allocate(omegani(0:nl/2-1,0:nl/2-1))
 
-  call rotate_array()
+  call rotate_array_f()
   call rotate_calc( nl/2, 'r', (/prim(1)-1,prim(2),prim(3),prim(4),prim(5)/),  &
   &                 omegar(0:prim(5)-1,0:prim(5)-1),  &
   &                 omeganr(0:nl/2-1,0:nl/2-1) )
