@@ -1,3 +1,7 @@
+!---------------------------------------------------------------
+! Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
+!---------------------------------------------------------------
+
 module Matrix_Calc
 !  ׻˹Ԥ롼.
 !  ͭͷ׻չ׻, ϢΩεʤɤԤ.
@@ -9,11 +13,27 @@ module Matrix_Calc
   public :: SOR_Gau_Sei
   public :: SOR_Jacobi_algebra
   public :: gausss
+  public :: tri_gauss
   public :: schumit_norm
+  public :: mat_dot
+  public :: Householder
+  public :: QR_method
   private :: errata
   private :: Pivot_part
   private :: vec_dot
+  private :: sum_sq
 
+interface gausss
+
+  module procedure gausssf, gausssd
+
+end interface gausss
+
+interface tri_gauss
+
+  module procedure tri_gaussf, tri_gaussd
+
+end interface tri_gauss
 
 interface trans_mat
 !  module Matrix_Calc
@@ -107,6 +127,36 @@ end function
 !-----------------------------------------------------------------
 !-----------------------------------------------------------------
 
+subroutine mat_dot( a, b, c )
+! ƱΤѤ׻ (a * b = c).
+  implicit none
+  real, dimension(:,:), intent(in) :: a, b
+  real, dimension(:,:), intent(inout) :: c
+  integer :: i, j, k, l, m, n
+
+  m=size(a,1)
+  n=size(a,2)
+
+  if(n/=size(b,1).or.m/=size(b,2).or.m/=size(c,1).or.m/=size(c,2))then
+     write(*,*) "ERROR (mat_dot) : when a * b = c, a=mxn -> b=nxm, c=mxm"
+     write(*,*) "STOP."
+     stop
+  end if
+
+  do j=1,m
+     do i=1,m
+        c(i,j)=0.0
+        do k=1,n
+           c(i,j)=c(i,j)+a(i,k)*b(k,j)
+        end do
+     end do
+  end do
+
+end subroutine mat_dot
+
+!-----------------------------------------------------------------
+!-----------------------------------------------------------------
+
 subroutine invert_mat( ax, xx )
 ! ξõˡĥ, չ׻.
 ! Ūˤ, դ b ñ̥٥ȥ 1 Ĥ,õ̤Υ٥ȥ¤٤
@@ -144,14 +194,14 @@ end subroutine invert_mat
 !  ʲ, ϢΩ 1 ε롼
 !-----------------------------------------------------------------
 
-subroutine gausss( c, d, x )
+subroutine gausssf( c, d, x )
 ! ʬԥܥåդξõˡ
   implicit none
   real, intent(in) :: d(:)
-  real, intent(in) :: c(size(d),size(d))  !  ( 1 ǤԤɽ)
+  real, intent(in) :: c(size(d),size(d))  !  ( 1 Ǥֹԡפɽ)
   real, intent(inout) :: x(size(d))
   real :: b(size(d))
-  real :: a(size(d),size(d))  !  ( 1 ǤԤɽ)
+  real :: a(size(d),size(d))  !  ( 1 Ǥפɽ)
   real :: s, pivotb
   real :: pivot(size(d)+1)
   integer :: piv, i, j, k, nmax
@@ -160,7 +210,7 @@ subroutine gausss( c, d, x )
 
   do k=1,nmax
      do j=1,nmax
-        a(j,k)=c(j,k)
+        a(k,j)=c(j,k)   ! ǹؤ (׻ΤϤǹԤ)
      end do
      b(k)=d(k)
   end do
@@ -215,7 +265,172 @@ subroutine gausss( c, d, x )
      x(i)=s
   end do
 
-end subroutine gausss
+end subroutine gausssf
+
+!------------------------------------------
+
+subroutine gausssd( c, d, x )
+! ʬԥܥåդξõˡ
+  implicit none
+  double precision, intent(in) :: d(:)
+  double precision, intent(in) :: c(size(d),size(d))  !  ( 1 Ǥֹԡפɽ)
+  double precision, intent(inout) :: x(size(d))
+  double precision :: b(size(d))
+  double precision :: a(size(d),size(d))  !  ( 1 Ǥפɽ)
+  double precision :: s, pivotb
+  double precision :: pivot(size(d)+1)
+  integer :: piv, i, j, k, nmax
+
+  nmax=size(b)
+
+  do k=1,nmax
+     do j=1,nmax
+        a(k,j)=c(j,k)   ! ǹؤ (׻ΤϤǹԤ)
+     end do
+     b(k)=d(k)
+  end do
+
+!-- ʾõ ---
+!-- A(I,J) ʾõ ---
+  do k=1,nmax-1
+!-- PIVOT  ---
+!-- ޤ, I ʬκͤꤹ ---
+     piv=k
+     do i=k+1,nmax
+         if(abs(a(i,k)).gt.abs(a(piv,k)))then
+            piv=i
+         end if
+     end do
+!-- ޤǤ, ͤꤵ ---
+!-- ʲ, ͤȤʬιԤؤ ---
+     do j=k,nmax
+        pivot(j)=a(k,j)
+        a(k,j)=a(piv,j)
+        a(piv,j)=pivot(j)
+     end do
+     pivotb=b(k)
+     b(k)=b(piv)
+     b(piv)=pivotb
+!-- PIVOT ޤ ---
+     do i=k+1,nmax
+        a(k,i)=a(k,i)/a(k,k)
+     end do
+     b(k)=b(k)/a(k,k)
+     a(k,k)=1.0d0
+
+     do j=k+1,nmax
+        do i=k+1,nmax
+            a(j,i)=a(j,i)-a(k,i)*a(j,k)
+        end do
+        b(j)=b(j)-b(k)*a(j,k)
+        a(j,k)=0.0d0
+     end do
+  end do
+
+  b(nmax)=b(nmax)/a(nmax,nmax)
+  a(nmax,nmax)=1.0d0
+
+!-- X(I) θ
+  x(nmax)=b(nmax)
+  do i=nmax-1,1,-1
+     s=b(i)
+     do j=i+1,nmax
+        s=s-a(i,j)*x(j)
+     end do
+     x(i)=s
+  end do
+
+end subroutine gausssd
+
+!------------------------------------------
+
+subroutine tri_gaussf( c, d, x )
+! гѹǥξõˡ
+  implicit none
+  real, intent(in) :: d(:)
+  real, intent(in) :: c(size(d),size(d))  !  ( 1 Ǥֹԡפɽ)
+  real, intent(inout) :: x(size(d))
+  real :: b(size(d))
+  real :: a(size(d),size(d))  !  ( 1 Ǥפɽ)
+  real :: s
+  integer :: i, j, k, nmax
+
+  nmax=size(b)
+
+  do k=1,nmax
+     do j=1,nmax
+        a(k,j)=c(j,k)   ! ǹؤ (׻ΤϤǹԤ)
+     end do
+     b(k)=d(k)
+  end do
+
+!-- ʾõ ---
+!-- A(I,J) ʾõ ---
+  do k=1,nmax-1
+     a(k,k+1)=a(k,k+1)/a(k,k)
+     b(k)=b(k)/a(k,k)
+     a(k,k)=1.0
+
+     a(k+1,k+1)=a(k+1,k+1)-a(k,k+1)*a(k+1,k)
+     b(k+1)=b(k+1)-b(k)*a(k+1,k)
+     a(k+1,k)=0.0
+  end do
+
+  b(nmax)=b(nmax)/a(nmax,nmax)
+  a(nmax,nmax)=1.0
+
+!-- X(I) θ
+  x(nmax)=b(nmax)
+  do i=nmax-1,1,-1
+     x(i)=b(i)-a(i,i+1)*x(i+1)
+  end do
+
+end subroutine tri_gaussf
+
+!------------------------------------------
+
+subroutine tri_gaussd( c, d, x )
+! гѹǥξõˡ
+  implicit none
+  double precision, intent(in) :: d(:)
+  double precision, intent(in) :: c(size(d),size(d))  !  ( 1 Ǥֹԡפɽ)
+  double precision, intent(inout) :: x(size(d))
+  double precision :: b(size(d))
+  double precision :: a(size(d),size(d))  !  ( 1 Ǥפɽ)
+  double precision :: s
+  integer :: i, j, k, nmax
+
+  nmax=size(b)
+
+  do k=1,nmax
+     do j=1,nmax
+        a(k,j)=c(j,k)   ! ǹؤ (׻ΤϤǹԤ)
+     end do
+     b(k)=d(k)
+  end do
+
+!-- ʾõ ---
+!-- A(I,J) ʾõ ---
+  do k=1,nmax-1
+     a(k,k+1)=a(k,k+1)/a(k,k)
+     b(k)=b(k)/a(k,k)
+     a(k,k)=1.0d0
+
+     a(k+1,k+1)=a(k+1,k+1)-a(k,k+1)*a(k+1,k)
+     b(k+1)=b(k+1)-b(k)*a(k+1,k)
+     a(k+1,k)=0.0d0
+  end do
+
+  b(nmax)=b(nmax)/a(nmax,nmax)
+  a(nmax,nmax)=1.0d0
+
+!-- X(I) θ
+  x(nmax)=b(nmax)
+  do i=nmax-1,1,-1
+     x(i)=b(i)-a(i,i+1)*x(i+1)
+  end do
+
+end subroutine tri_gaussd
 
 !------------------------------------------
 
@@ -223,7 +438,7 @@ subroutine LU_devs( a, b, x, itermax )
 !-- LU ʬ׻륵֥롼 ---
   implicit none
   real, intent(inout) :: b(:)  ! դΥ٥ȥ
-  real, intent(inout) :: a(size(b),size(b))  !  ( 1 ǤԤɽ)
+  real, intent(inout) :: a(size(b),size(b))  !  ( 1 Ǥֹԡפɽ)
   real, intent(inout) :: x(size(b))  ! 
   integer, intent(in) :: itermax  ! ȿβ
   real :: d(size(b),size(b)), r(size(b)), y(size(b))
@@ -246,7 +461,7 @@ subroutine LU_devs( a, b, x, itermax )
 
   do i=1,nmax
      do j=1,nmax
-        d(i,j)=a(i,j)
+        d(i,j)=a(j,i)   ! ǹؤ (׻ΤϤǹԤ)
      end do
 
      ip(i)=i
@@ -332,7 +547,7 @@ subroutine LU_devs( a, b, x, itermax )
      do i=1,nmax
         s3=0.0
         do j=1,nmax
-           s3=s3+a(i,j)*x(j)
+           s3=s3+a(j,i)*x(j)  ! a  1 ܤ
         end do
         r(i)=b(i)-s3
      end do
@@ -404,14 +619,13 @@ subroutine Gau_Sei(a, b, eps, x)
 
      do j=1,nx
         if(j/=i)then
-           xn=xn+a(i,j)*x(j)
+           xn=xn+a(j,i)*x(j)
         end if
      end do
 
      xn=(b(i)-xn)/a(i,i)
 
      err=errata(x(i),xn,1)
-write(*,*) "err_max", x(i), nx, err_max,err
 
      if(err_max<=err)then
         err_max=err
@@ -429,7 +643,7 @@ write(*,*) "err_max", x(i), nx, err_max,err
 
         do j=1,nx
            if(j/=i)then
-              xn=xn+a(i,j)*x(j)
+              xn=xn+a(j,i)*x(j)
            end if
         end do
 
@@ -477,14 +691,13 @@ subroutine Jacobi_algebra(a, b, eps, x)
 
      do j=1,nx
         if(j/=i)then
-           y(i)=y(i)+a(i,j)*x(j)
+           y(i)=y(i)+a(j,i)*x(j)
         end if
      end do
 
      y(i)=(b(i)-y(i))/a(i,i)
 
      err=errata(x(i),y(i),1)
-write(*,*) "err_max", x(i), nx, err_max,err
 
      if(err_max<=err)then
         err_max=err
@@ -503,7 +716,7 @@ write(*,*) "err_max", x(i), nx, err_max,err
 
         do j=1,nx
            if(j/=i)then
-              y(i)=y(i)+a(i,j)*x(j)
+              y(i)=y(i)+a(j,i)*x(j)
            end if
         end do
 
@@ -560,7 +773,7 @@ subroutine SOR_Gau_Sei(a, b, eps, accel, x)
 
      do j=1,nx
         if(j/=i)then
-           xn=xn+a(i,j)*x(j)
+           xn=xn+a(j,i)*x(j)
         end if
      end do
 
@@ -568,7 +781,6 @@ subroutine SOR_Gau_Sei(a, b, eps, accel, x)
      xn=x(i)+accel*(xn-x(i))
 
      err=errata(x(i),xn,1)
-write(*,*) "err_max", x(i), nx, err_max,err
 
      if(err_max<=err)then
         err_max=err
@@ -586,7 +798,7 @@ write(*,*) "err_max", x(i), nx, err_max,err
 
         do j=1,nx
            if(j/=i)then
-              xn=xn+a(i,j)*x(j)
+              xn=xn+a(j,i)*x(j)
            end if
         end do
 
@@ -644,14 +856,13 @@ subroutine SOR_Jacobi_algebra(a, b, eps, accel, x)
 
      do j=1,nx
         if(j/=i)then
-           y(i)=y(i)+a(i,j)*x(j)
+           y(i)=y(i)+a(j,i)*x(j)
         end if
      end do
 
      y(i)=(b(i)-y(i))/a(i,i)
 
      err=errata(x(i),y(i),1)
-write(*,*) "err_max", x(i), nx, err_max,err
 
      if(err_max<=err)then
         err_max=err
@@ -670,7 +881,7 @@ write(*,*) "err_max", x(i), nx, err_max,err
 
         do j=1,nx
            if(j/=i)then
-              y(i)=y(i)+a(i,j)*x(j)
+              y(i)=y(i)+a(j,i)*x(j)
            end if
         end do
 
@@ -727,7 +938,7 @@ subroutine eigenvalue_power( a, eps, eigenval, eigenvec )
      do i=1,nx
         y(i)=0.0  ! ν
         do j=1,nx
-           y(i)=y(i)+a(i,j)*x(j)
+           y(i)=y(i)+a(j,i)*x(j)
         end do
      end do
 
@@ -780,8 +991,8 @@ subroutine Jacobi_eigen( a, lambda, eps )
 
   do j=1,n
      do i=1,n
-        tmp_a(i,j)=a(i,j)
-        new_a(i,j)=a(i,j)
+        tmp_a(i,j)=a(j,i)  ! ؤ
+        new_a(i,j)=a(j,i)  ! ؤ
      end do
   end do
 
@@ -854,69 +1065,333 @@ end subroutine Jacobi_eigen
 !-----------------------------------------
 !-----------------------------------------
 
-!-- ʲ, ȯ !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-subroutine QR_dev( a, lambda, eps, method )
+subroutine QR_method( a, lambda, p, eps, sym_opt )
 ! QR ʬˡѤƹ a ͭͤ׻롼.
+! ʸ (2007) 4.8.
+! ʲ줿ͭ٥ȥ褦ˤƤ뤬, ͤΤɤ̤ǧ.
+! ͭͤ˴ؤƤ꤫ǧѤ.
+! ׻ȤƤ,
+! 1. Householder Ѵ
+! 2. ľη׻
+! 3. QR ʬ
+! 4. QR 
   implicit none
   real, intent(in) :: a(:,:)  ! ͭͤ [ 1 Ǥ]
   real, intent(inout) :: lambda(size(a,1))  ! a θͭ
-  real, intent(in), optional :: eps  ! ȿˡμ«
-  character(1), intent(in), optional :: method  ! a оιоι󤫤Υե饰
-                ! ǥեȤоι.
-                ! 'S' : оι -> гѲԤ,
-                ! 'A' : оι -> Householder ѴԤ.
-  integer :: i, j, n
-  real, dimension(size(a,1),size(a,2)) :: tmp_a, new_a
+  real, intent(inout), optional :: p(size(a,1),size(a,2))  ! ͭ٥ȥ.
+                       ! ֤ˤ, Householder Ѵν֤٤ʤ.
+  real, intent(in), optional :: eps  ! ȿˡμ« default = 1.0e-5
+  logical, intent(in), optional :: sym_opt  ! a оι.
+                              ! .true. = оι, default = .false.
+  integer :: i, j, k, n, counter
+  real :: dia, s, t, err, err_max, tmp1, tmp2, mu
+  real, dimension(size(a,1),size(a,2)) :: a0, q, r, b, tmp_p, tmp_pp
+  logical :: sym_link, pconv_flag
 
   n=size(a,1)
 
-!-- intent(in) 񤭴ʤΤ, ֤.
-  do j=1,n
-     do i=1,n
-        tmp_a(i,j)=a(i,j)
-        new_a(i,j)=a(i,j)
-     end do
-  end do
-
-!-- ʲ, Ѵ.
-  if(present(method))then
-     if(method=='S')then   ! гѲ.
-     else   ! Householder Ѵ.
-     end if
-  else  ! Householder Ѵ.
-     
+  if(present(sym_opt))then
+     sym_link=sym_opt
+  else
+     sym_link=.false.
   end if
 
+  if(present(p))then
+     pconv_flag=.true.
+     p=0.0
+  else
+     pconv_flag=.false.
+  end if
+
+  if(present(eps))then
+     err_max=eps
+  else
+     err_max=1.0e-5
+  end if
+
+!-- 1. Householder Ѵ.
+  if(pconv_flag.eqv..true.)then
+     call Householder( a, a0, p=tmp_p, sym_opt=sym_link )
+  else
+     call Householder( a, a0, sym_opt=sym_link )
+  end if
+
+  counter=n
+
+  do while(counter>1)
+     err=err_max
+     do while(err_max<=err)
+        q=0.0   ! QR ʬ Q_k 
+
+        do k=1,n
+           q(k,k)=1.0    ! ǽñ̹Ȥƽ
+        end do
+
+        mu=a0(counter,counter)
+        a0(counter,counter)=0.0
+        do k=1,counter-1
+           a0(k,k)=a0(k,k)-mu    ! moving original point
+        end do
+
 !-- QR ʬԤ.
-  do j=1,n
-     do i=1,n
+        do k=1,counter-1
+           dia=sqrt(abs(a0(k,k))**2+abs(a0(k+1,k))**2)
+           s=a0(k+1,k)/dia
+           t=a0(k+1,k)/(dia+a0(k,k))
+
+           a0(k+1,k)=0.0
+           a0(k,k)=dia
+
+           do j=k+1,counter
+              tmp1=a0(k,j)
+              tmp2=a0(k+1,j)
+              a0(k,j)=tmp1+s*(tmp2-t*tmp1)
+              a0(k+1,j)=tmp2-s*(tmp1+t*tmp2)
+           end do
+
+           do i=1,counter
+              tmp1=q(i,k)
+              tmp2=q(i,k+1)
+              q(i,k)=tmp1+s*(tmp2-t*tmp1)
+              q(i,k+1)=tmp2-s*(tmp1+t*tmp2)
+           end do
+        end do
+
+        do j=1,counter
+           do i=1,counter
+              r(i,j)=a0(i,j)     ! QR ʬ줿廰ѹ R
+           end do
+        end do
+
+!-- QR εѤ׻. A=RQ , R Ͼ廰, Q ϥإå٥륰ʤΤ,
+!-- a_{i,j} = R_{i,k} * Q_{k,j} + R_{i,k+1} * Q_{k+1,j}, (k=i)
+!-- , Q ž֤Ƿ׻Τ, Q=qT Ȥ, Q_{k,j} = q_{j,k}.
+!-- a_{i,j} = R_{i,k} * q_{j,k} + R_{i,k+1} * q_{j,k+1}, (k=i)
+
+        call mat_dot( r, q, a0 )
+
+!-- mooving grid
+        do i=1,counter
+           a0(i,i)=a0(i,i)+mu
+        end do
+
+!-- «Ƚ
+        if(abs(err_max)>abs(a0(counter,counter-1)))then
+           err=abs(a0(counter,counter-1))
+        end if
+
+        if(pconv_flag.eqv..true.)then
+           call mat_dot( q, tmp_p, tmp_pp )
+           tmp_p(1:n,1:n)=tmp_pp(1:n,1:n)
+        end if
+
      end do
+
+     lambda(counter)=a0(counter,counter)
+     counter=counter-1
+
   end do
 
+  if(pconv_flag.eqv..true.)then
+     p(1:n,1:n)=tmp_p(1:n,1:n)
+  end if
 
+  lambda(1)=a0(1,1)
 
-end subroutine QR_dev
+end subroutine QR_method
 
 !-----------------------------------------
 !-----------------------------------------
 
-!-- ʲ, ȯ !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-subroutine eigen_power_all( a, lambda, eigen_vec, eps )
-!  ٤ˡѤ, a ˤĤƤƤθͭͤȤȼͭ٥ȥ
-!  ׻롼.
+subroutine Householder( a, b, p, sym_opt )
+! ϥۥѴѤ, Ǥռоιإå٥륰Ѵ.
+! , Υ롼ǤϷ׻®٤θ, гѤξ廰ʬϷ׻ʤ.
+! оιξ, 廰ʬϲʬƱˤʤΤ,
+! 廰ʬͤ, b βʬФ褤.
+! ͤ (2007)  4.5.2.
   implicit none
-  real, intent(in) :: a(:,:)  ! ͭͤ [ 1 Ǥ]
-  real, intent(inout) :: lambda(size(a,1))  ! a θͭ
-  real, intent(inout) :: eigen_vec(size(a,1),size(a,2))  ! lambda(i) бͭ٥ȥ, Ǥͭ lambda(i) бƤ.
-  real, intent(in), optional :: eps  ! ȿˡμ«
-  integer :: n
+  real, intent(in) :: a(:,:)   ! Ǥդμоι
+  real, intent(inout) :: b(size(a,1),size(a,2))  ! إå٥륰
+  real, intent(inout), optional :: p(size(a,1),size(a,2))  ! ѴѤľ
+                                   ! ֤, Ѥ˷׻ԤΤ,
+                                   ! ׻֤٤ʤ뤳Ȥ.
+  logical, intent(in), optional :: sym_opt   ! a оι󤫤ɤ.
+                       ! default = .false. (оΤǤϤʤ)
+                       ! .true. ξ, b оʬ򤽤Τޤ֤.
+                       ! ׻®٤θ̾оʬ׻ʤ.
+  integer :: i, j, k, n
+  real :: coe, s, t, tmp
+  real, allocatable, dimension(:) :: w, pconv, qconv
+  real, dimension(size(a,1),size(a,2)) :: tmp_p, tmp_pp
+  logical :: sym_link, pconv_flag
 
   n=size(a,1)
 
+  if(present(sym_opt))then
+     sym_link=sym_opt
+  else
+     sym_link=.false.
+  end if
 
+  if(present(p))then
+     pconv_flag=.true.
+     p=0.0
+  else
+     pconv_flag=.false.
+  end if
 
+  if(size(a,1)/=size(a,2))then
+     write(*,*) "*** ERROR (Householder) ***"
+     write(*,*) "Input array must be square matrix. STOP."
+     stop
+  end if
 
-end subroutine
+  b(:,:)=a(:,:)
+
+  allocate(w(n))
+  allocate(pconv(n))
+  allocate(qconv(n))
+
+  if(sym_link.eqv..true.)then
+     do j=1,n-2
+        pconv=0.0
+        qconv=0.0
+        w=0.0
+        tmp_p=0.0
+     !-- Ѵκ
+        t=sum_sq( b(j+1:n,j) )
+        if(b(j+1,j)>0.0)then
+           s=sqrt(t)
+        else
+           s=-sqrt(t)
+        end if
+
+        w(1:j)=0.0
+        w(j+1)=b(j+1,j)+s
+        coe=1.0/(t+b(j+1,j)*s)   ! η (2007)  c ǤϤʤ, 2c Ǥ.
+        w(j+2:n)=b(j+2:n,j)
+
+        do k=j+1,n
+           do i=j+1,n
+              pconv(k)=pconv(k)+b(k,i)*w(i)
+           end do
+           pconv(k)=coe*pconv(k)
+        end do
+
+        do k=j+1,n
+           do i=j+1,n
+              qconv(k)=qconv(k)+pconv(i)*w(i)
+           end do
+           qconv(k)=pconv(k)-0.5*coe*qconv(k)*w(k)
+        end do
+
+     !-- ѴˤѴ
+
+        b(j+1,j)=-s
+        b(j+2:n,j)=0.0
+
+        do k=j+1,n
+           do i=j+1,n
+              b(i,k)=b(i,k)-qconv(i)*w(k)-w(i)*qconv(k)
+           end do
+        end do
+
+        if(pconv_flag.eqv..true.)then
+           do k=j+1,n   ! w  j+1 ʾʬΤ󥼥.
+              do i=j+1,n
+                 tmp_p(i,k)=-coe*w(i)*w(k)
+              end do
+           end do
+           do k=1,n
+              tmp_p(k,k)=1.0+tmp_p(k,k)
+           end do
+
+           if(j==1)then
+              p(1:n,1:n)=tmp_p(1:n,1:n)
+           else
+              call mat_dot( p, tmp_p, tmp_pp )
+              p(1:n,1:n)=tmp_pp(1:n,1:n)
+           end if
+
+        end if
+
+     end do
+
+     do k=1,n
+        do j=k,n
+           b(k,j)=b(j,k)
+        end do
+     end do
+
+  else   ! оιξ (ޤбƤʤ.)
+
+     do j=1,n-2
+        pconv=0.0
+        qconv=0.0
+        w=0.0
+        tmp_p=0.0
+     !-- Ѵκ
+        t=sum_sq( b(j+1:n,j) )
+        if(b(j+1,j)>0.0)then
+           s=sqrt(t)
+        else
+           s=-sqrt(t)
+        end if
+
+        w(1:j)=0.0
+        w(j+1)=b(j+1,j)+s
+        coe=1.0/(t+b(j+1,j)*s)   ! η (2007)  c ǤϤʤ, 2c Ǥ.
+        w(j+2:n)=b(j+2:n,j)
+
+        do k=j+1,n
+           do i=j+1,n
+              pconv(k)=pconv(k)+b(k,i)*w(i)
+           end do
+           pconv(k)=coe*pconv(k)
+        end do
+
+        do k=j+1,n
+           do i=j+1,n
+              qconv(k)=qconv(k)+pconv(i)*w(i)
+           end do
+           qconv(k)=pconv(k)-0.5*coe*qconv(k)*w(k)
+        end do
+
+     !-- ѴˤѴ
+
+        b(j+1,j)=-s
+        b(j+2:n,j)=0.0
+
+        do k=j+1,n
+           do i=j+1,n
+              b(i,k)=b(i,k)-qconv(i)*w(k)-w(i)*qconv(k)
+           end do
+        end do
+
+        if(pconv_flag.eqv..true.)then
+           do k=j+1,n   ! w  j+1 ʾʬΤ󥼥.
+              do i=j+1,n
+                 tmp_p(i,k)=-coe*w(i)*w(k)
+              end do
+           end do
+           do k=1,n
+              tmp_p(k,k)=1.0+tmp_p(k,k)
+           end do
+
+           if(j==1)then
+              p(1:n,1:n)=tmp_p(1:n,1:n)
+           else
+              call mat_dot( p, tmp_p, tmp_pp )
+              p(1:n,1:n)=tmp_pp(1:n,1:n)
+           end if
+
+        end if
+
+     end do
+
+  end if
+
+end subroutine Householder
 
 !-----------------------------------------
 !-----------------------------------------
@@ -926,6 +1401,8 @@ subroutine schumit_norm( u, v )
 !  1 Ǥ٥ȥγʬ,  2 ʬ٥ȥ뷲 1 ٥ȥɽ.
 ! Ĥޤ, u(i,j)  j ܥ٥ȥ i ʬȤȤɽ.
 ! ɽʤ, ĥ٥ȥ򲣤¤٤.
+! ܥ롼ϥߥåȤľ򲽤ܤ˷׻ƤΤ,
+! ׻®٤Ū٤餷.
   implicit none
   real, intent(in) :: u(:,:)  ! ľ򲽤Υ٥ȥ
   real, intent(inout) :: v(size(u,1),size(u,2))  ! ľ򲽸Υ٥ȥ
@@ -963,6 +1440,27 @@ subroutine schumit_norm( u, v )
 
 end subroutine schumit_norm
 
+!-----------------------------------------
+!-----------------------------------------
+
+!-- ʲ, ȯ !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+subroutine eigen_power_all( a, lambda, eigen_vec, eps )
+!  ٤ˡѤ, a ˤĤƤƤθͭͤȤȼͭ٥ȥ
+!  ׻롼.
+  implicit none
+  real, intent(in) :: a(:,:)  ! ͭͤ [ 1 Ǥ]
+  real, intent(inout) :: lambda(size(a,1))  ! a θͭ
+  real, intent(inout) :: eigen_vec(size(a,1),size(a,2))  ! lambda(i) бͭ٥ȥ, Ǥͭ lambda(i) бƤ.
+  real, intent(in), optional :: eps  ! ȿˡμ«
+  integer :: n
+
+  n=size(a,1)
+
+
+
+
+end subroutine
+
 !-----------------------------------------
 ! ʲ, private °դ롼
 !-----------------------------------------
@@ -973,24 +1471,31 @@ subroutine Pivot_part( a, b )
   real, intent(inout) :: b(:)  ! ԥܥåƥ󥰤ȤȤ֤뱦ե٥ȥ
   real, intent(inout) :: a(size(b),size(b))  ! ԥܥåƥ󥰤Ԥ ( 1 ǤԤб)
   real :: tmp(size(b))  ! ؤΰ
+  real :: tmpa(size(b),size(b))  ! 1 Ǥפб)
   real :: tmpv  ! ؤȼ b(i) 򤵤ѿ
   integer :: i, j, k
   integer :: nx
 
   nx=size(b)
 
+  do j=1,nx
+     do i=1,nx
+        tmpa(j,i)=a(i,j)  ! ǹԤؤ
+     end do
+  end do
+
   do i=1,nx
-     if(a(i,i)==0.0)then
+     if(tmpa(i,i)==0.0)then
         do j=1,nx  !  do  j=1 ֤Ԥõ
 ! ʲȽ̼ϼ̤
 ! a(i,i)==0 ʤΤ, j ܤ i 󤬥ǤʤԤõ a(j,i)/=0.
 !  j ܤؤ뤬, ؤȤ,  i ܤˤä j ؤȤ
 ! j ܤгʬ a(j,j) ˤʤʤ褦 a(i,j)/=0  j ꤹ.
-           if(a(i,j)/=0.0.and.a(j,i)/=0.0.and.i/=j)then
+           if(tmpa(i,j)/=0.0.and.tmpa(j,i)/=0.0.and.i/=j)then
               do k=1,nx  ! ºݤؤ
-                 tmp(k)=a(i,k)
-                 a(i,k)=a(j,k)
-                 a(j,k)=tmp(k)
+                 tmp(k)=tmpa(i,k)
+                 tmpa(i,k)=tmpa(j,k)
+                 tmpa(j,k)=tmp(k)
               end do
               tmpv=b(i)
               b(i)=b(j)
@@ -1001,6 +1506,12 @@ subroutine Pivot_part( a, b )
      end if
   end do
 
+  do j=1,nx
+     do i=1,nx
+        a(j,i)=tmpa(i,j)  ! ǹԤؤ
+     end do
+  end do
+
 end subroutine Pivot_part
 
 !-----------------------------------------
@@ -1046,12 +1557,27 @@ real function vec_dot( u, v )
   return
 end function
 
-
-
-!-----------------------------------------
 !-----------------------------------------
 
-!-----------------------------------------
+real function sum_sq( a )
+! Ǥդ 1 Ǥγʬ 2 ¤׻.
+  implicit none
+  real, intent(in) :: a(:)
+  integer :: i, n
+  real :: tmp
+
+  n=size(a)
+  tmp=0.0
+
+  do i=1,n
+     tmp=tmp+a(i)**2
+  end do
+
+  sum_sq=tmp
+
+  return
+end function sum_sq
+
 !-----------------------------------------
 
 end module Matrix_Calc
