@@ -1,5 +1,5 @@
 !---------------------------------------------------------------
-! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+! Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !---------------------------------------------------------------
 
 module Thermo_Advanced_Function
@@ -147,7 +147,7 @@ real function precip_water( p, qv, undef, dl )  ! Ĺ߿̤׻. ñ̤ [kg/m^2]
   real, intent(in) :: qv(size(p))  !  [kg/kg]
   real, intent(in), optional :: undef  ! undef
   integer, intent(in), optional :: dl  ! ǥХå٥
-  integer :: nx, i
+  integer :: nx, i, ibot, itop
   real, dimension(size(p)) :: tmp_p, tmp_qv
   real :: precip
 
@@ -160,7 +160,23 @@ real function precip_water( p, qv, undef, dl )  ! Ĺ߿̤׻. ñ̤ [kg/m^2]
   end do
 
   if(present(undef))then
-     call rectangle_int( tmp_p, tmp_qv, tmp_p(1), tmp_p(nx), precip, undef )
+     ibot=1
+     itop=nx
+     do i=1,nx
+        if(tmp_p(i)/=undef)then
+           ibot=i
+           exit
+        end if
+     end do
+     do i=nx,1,-1
+        if(tmp_p(i)/=undef)then
+           itop=i
+           exit
+        end if
+     end do
+     call rectangle_int( tmp_p(ibot:itop), tmp_qv(ibot:itop),  &
+  &                      tmp_p(ibot), tmp_p(itop),  &
+  &                      precip, undef )
   else
      call rectangle_int( tmp_p, tmp_qv, tmp_p(1), tmp_p(nx), precip )
   end if
@@ -192,13 +208,14 @@ real function CAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, opt, dl )
   real, intent(in), optional :: undeff
   integer, intent(in), optional :: opt  ! LNB ׻κݤΥץ (z_LNB )
   integer, intent(in), optional :: dl  ! ǥХå٥
-  integer :: nx, i, iz_LFC, iz_LNB, option
+  integer :: nx, i, iz_LFC, iz_LNB, option, iundef
   real :: PLFC, TLFC, PLNB, ZLFC, ZLNB
   real :: tmp(size(tmp_p)), t_par(size(tmp_p))
   real :: inv_tmp(size(tmp_p)), inv_p(size(tmp_p))
   real :: undef
 
   t_par=0.0
+  iundef=0
 
   nx=size(tmp_p)
 
@@ -221,59 +238,61 @@ real function CAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, opt, dl )
   ZLNB=z_LNB( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv, opt=option, undeff=undef )
 
   if(ZLFC/=undef.and.ZLNB/=undef)then
-     call interpo_search_1d( tmp_z, ZLFC, iz_LFC, int(undef) )
-     call interpo_search_1d( tmp_z, ZLNB, iz_LNB, int(undef) )
+     call interpo_search_1d( tmp_z, ZLFC, iz_LFC, iundef )
+     call interpo_search_1d( tmp_z, ZLNB, iz_LNB, iundef )
      if(iz_LFC>=nx)then
-        iz_LFC=int(undef)
+        iz_LFC=iundef
      end if
      if(iz_LNB>=nx)then
-        iz_LNB=int(undef)
+        iz_LNB=iundef
      end if
   else
-     iz_LFC=int(undef)
-     iz_LNB=int(undef)
+     iz_LFC=iundef
+     iz_LNB=iundef
   end if
 
-  if(iz_LFC/=int(undef).and.iz_LNB/=int(undef))then
+  if(iz_LFC/=iundef.and.iz_LNB/=iundef)then
 
-  if(tmp_t(iz_LFC)/=undef.and.tmp_qv(iz_LFC)/=undef.and.tmp_p(iz_LFC)/=undef  &
-  &  .and.tmp_t(iz_LFC+1)/=undef.and.tmp_qv(iz_LFC+1)/=undef.and.tmp_p(iz_LFC+1)/=undef  &
-  &  .and.tmp_t(iz_LNB)/=undef.and.tmp_qv(iz_LNB)/=undef.and.tmp_p(iz_LNB)/=undef  &
-  &  .and.tmp_t(iz_LNB+1)/=undef.and.tmp_qv(iz_LNB+1)/=undef.and.tmp_p(iz_LNB+1)/=undef)then
-     call interpolation_1d( (/tmp_z(iz_LFC), tmp_z(iz_LFC+1)/),  &
-     &    (/tmp_p(iz_LFC), tmp_p(iz_LFC+1)/), ZLFC, PLFC )
-     call interpolation_1d( (/tmp_z(iz_LNB), tmp_z(iz_LNB+1)/),  &
-     &    (/tmp_p(iz_LNB), tmp_p(iz_LNB+1)/), ZLNB, PLNB )
+     if(tmp_t(iz_LFC)/=undef.and.tmp_qv(iz_LFC)/=undef.and.  &
+  &     tmp_p(iz_LFC)/=undef.and.tmp_t(iz_LFC+1)/=undef.and.  &
+  &     tmp_qv(iz_LFC+1)/=undef.and.tmp_p(iz_LFC+1)/=undef.and.  &
+  &     tmp_t(iz_LNB)/=undef.and.tmp_qv(iz_LNB)/=undef.and.  &
+  &     tmp_p(iz_LNB)/=undef.and.tmp_t(iz_LNB+1)/=undef.and.  &
+  &     tmp_qv(iz_LNB+1)/=undef.and.tmp_p(iz_LNB+1)/=undef)then
 
-     do i=1,nx
-        if(iz_LFC<=i.and.i<=iz_LNB+1)then
+        call interpolation_1d( (/tmp_z(iz_LFC), tmp_z(iz_LFC+1)/),  &
+  &          (/tmp_p(iz_LFC), tmp_p(iz_LFC+1)/), ZLFC, PLFC )
+        call interpolation_1d( (/tmp_z(iz_LNB), tmp_z(iz_LNB+1)/),  &
+  &          (/tmp_p(iz_LNB), tmp_p(iz_LNB+1)/), ZLNB, PLNB )
 
-           if(undef==tmp_t(i))then
-              tmp(i)=undef
+        do i=1,nx
+           if(iz_LFC<=i.and.i<=iz_LNB+1)then
+              if(undef==tmp_t(i))then
+                 tmp(i)=undef
+              else
+                 t_par(i)=moist_laps_temp( PLFC, TLFC, tmp_p(i) )
+                 tmp(i)=(t_par(i)-tmp_t(i))/tmp_p(i)
+              end if
            else
-              t_par(i)=moist_laps_temp( PLFC, TLFC, tmp_p(i) )
-              tmp(i)=(t_par(i)-tmp_t(i))/tmp_p(i)
+              tmp(i)=undef
            end if
-        else
-           tmp(i)=undef
-        end if
-     end do
+        end do
 
 !-- ʬΤ, ؤ
-     do i=1,nx
-        inv_tmp(i)=tmp(nx-i+1)
-        inv_p(i)=tmp_p(nx-i+1)
-     end do
+        do i=1,nx
+           inv_tmp(i)=tmp(nx-i+1)
+           inv_p(i)=tmp_p(nx-i+1)
+        end do
 
-     call rectangle_int( inv_p, inv_tmp, PLNB, PLFC, CAPE, undef )
+        call rectangle_int( inv_p, inv_tmp, PLNB, PLFC, CAPE, undef )
 
-     CAPE=CAPE*Rd
+        CAPE=CAPE*Rd
 
-  else
+     else
 
-     CAPE=undef
+        CAPE=undef
 
-  end if
+     end if
 
   else
 
@@ -392,13 +411,14 @@ real function CIN( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, dl )
   real, intent(in) :: z_ref  ! ѡ夲 [m]
   real, intent(in), optional :: undeff
   integer, intent(in), optional :: dl  ! ǥХå٥
-  integer :: nx, i, iz_LFC, iz_ref, iz_LCL
+  integer :: nx, i, iz_LFC, iz_ref, iz_LCL, iundef
   real :: PLFC, TLFC, ZLFC, PLCL, ZLCL, TLCL, p_ref, T_ref, qv_ref
   real :: tmp(size(tmp_p)), t_par(size(tmp_p))
   real :: inv_tmp(size(tmp_p)), inv_p(size(tmp_p))
   real :: undef
 
   t_par=0.0
+  iundef=0
 
   nx=size(tmp_p)
 
@@ -414,17 +434,17 @@ real function CIN( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, dl )
   TLFC=T_LFC( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
   ZLFC=z_LFC( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
 
-  call interpo_search_1d( tmp_z, ZLFC, iz_LFC, int(undef) )
-  call interpo_search_1d( tmp_z, z_ref, iz_ref, int(undef) )
+  call interpo_search_1d( tmp_z, ZLFC, iz_LFC, iundef )
+  call interpo_search_1d( tmp_z, z_ref, iz_ref, iundef )
 
   if(iz_LFC>=nx)then
-     iz_LFC=int(undef)
+     iz_LFC=iundef
   end if
   if(iz_ref>=nx)then
-     iz_ref=int(undef)
+     iz_ref=iundef
   end if
 
-  if(iz_LFC/=int(undef).and.iz_ref/=int(undef))then
+  if(iz_LFC/=iundef.and.iz_ref/=iundef)then
   if(tmp_t(iz_LFC)/=undef.and.tmp_qv(iz_LFC)/=undef.and.tmp_p(iz_LFC)/=undef  &
   &  .and.tmp_t(iz_LFC+1)/=undef.and.tmp_qv(iz_LFC+1)/=undef.and.tmp_p(iz_LFC+1)/=undef  &
   &  .and.tmp_t(iz_ref)/=undef.and.tmp_qv(iz_ref)/=undef.and.tmp_p(iz_ref)/=undef  &
@@ -443,13 +463,13 @@ real function CIN( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, dl )
      TLCL=TqvP_2_TLCL( t_ref, qv_ref, p_ref )
      ZLCL=z_LCL( z_ref, tmp_z, tmp_t, tmp_p, tmp_qv )
 
-     call interpo_search_1d( tmp_z, ZLCL, iz_LCL, int(undef) )
+     call interpo_search_1d( tmp_z, ZLCL, iz_LCL, iundef )
 
      if(iz_LCL>=nx)then
-        iz_LCL=int(undef)
+        iz_LCL=iundef
      end if
 
-     if(iz_LCL/=int(undef))then
+     if(iz_LCL/=iundef)then
      if(tmp_t(iz_LCL)/=undef.and.tmp_qv(iz_LCL)/=undef.and.tmp_p(iz_LCL)/=undef  &
   &  .and.tmp_t(iz_LCL+1)/=undef.and.tmp_qv(iz_LCL+1)/=undef.and.tmp_p(iz_LCL+1)/=undef)then
 
@@ -553,24 +573,26 @@ real function z_LCL( z_ref, z, temp, pres, qv, undeff, dl )  ! LCL ٤׻.
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   real ,intent(in), optional :: undeff
   integer, intent(in), optional :: dl  ! ǥХå٥
-  integer :: iz_ref
+  integer :: iz_ref, iundef
   real :: TLCL
   real :: T_ref, P_ref, qv_ref
   real :: undef
 
+  iundef=0
+
   if(present(undeff))then
      undef=undeff
   else
      undef=-999.0
   end if
 
-  call interpo_search_1d( z, z_ref, iz_ref, int(undef) )
+  call interpo_search_1d( z, z_ref, iz_ref, iundef )
 
   if(iz_ref>=size(z))then
-     iz_ref=int(undef)
+     iz_ref=iundef
   end if
 
-  if(iz_ref/=int(undef))then
+  if(iz_ref/=iundef)then
   if(temp(iz_ref)/=undef.and.qv(iz_ref)/=undef.and.pres(iz_ref)/=undef  &
   &  .and.temp(iz_ref+1)/=undef.and.qv(iz_ref+1)/=undef.and.pres(iz_ref+1)/=undef)then
      call interpolation_1d( (/z(iz_ref), z(iz_ref+1)/),  &
@@ -620,13 +642,14 @@ real function z_LFC( z_ref, z, temp, pres, qv, undeff, dl )  ! LFC ٤׻.
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   real, intent(in), optional :: undeff
   integer, intent(in), optional :: dl  ! ǥХå٥
-  integer :: i, nz, iz, iept
+  integer :: i, nz, iz, iept, iundef
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
   real :: undef
 
   nz=size(z)
   iept=0
+  iundef=0
 
   if(present(undeff))then
      undef=undeff
@@ -640,13 +663,13 @@ real function z_LFC( z_ref, z, temp, pres, qv, undeff, dl )  ! LFC ٤׻.
   end do
 
 !-- z_ref Ǥ̤׻.
-  call interpo_search_1d( z, z_ref, iz, int(undef) )
+  call interpo_search_1d( z, z_ref, iz, iundef )
 
   if(iz>=nz)then
-     iz=int(undef)
+     iz=iundef
   end if
 
-  if(iz/=int(undef))then
+  if(iz/=iundef)then
   if(temp(iz)/=undef.and.qv(iz)/=undef.and.pres(iz)/=undef  &
   &  .and.temp(iz+1)/=undef.and.qv(iz+1)/=undef.and.pres(iz+1)/=undef)then
 !-- iz  iz+1 Ǥ̤׻.
@@ -714,13 +737,14 @@ real function z_LNB( z_ref, z, temp, pres, qv, opt, undeff, dl )  ! LNB ٤׻.
   integer, intent(in), optional :: opt  ! ׻ˡΥץ
   real, intent(in), optional :: undeff
   integer, intent(in), optional :: dl  ! ǥХå٥
-  integer :: i, nz, iz, iept, counter
+  integer :: i, nz, iz, iept, counter, iundef
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
   real :: undef
 
   nz=size(z)
   iept=0
+  iundef=0
 
   if(present(undeff))then
      undef=undeff
@@ -734,13 +758,13 @@ real function z_LNB( z_ref, z, temp, pres, qv, opt, undeff, dl )  ! LNB ٤׻.
   end do
 
 !-- z_ref Ǥ̤׻.
-  call interpo_search_1d( z, z_ref, iz, int(undef) )
+  call interpo_search_1d( z, z_ref, iz, iundef )
 
   if(iz>=nz)then
-     iz=int(undef)
+     iz=iundef
   end if
 
-  if(iz/=int(undef))then
+  if(iz/=iundef)then
   if(temp(iz)/=undef.and.qv(iz)/=undef.and.pres(iz)/=undef  &
   &  .and.temp(iz+1)/=undef.and.qv(iz+1)/=undef.and.pres(iz+1)/=undef)then
 !-- iz  iz+1 Ǥ̤׻.
@@ -830,13 +854,14 @@ real function T_LFC( z_ref, z, temp, pres, qv, undeff, dl )  ! LFC ٤Ǥβ٤׻.
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   real, intent(in), optional :: undeff
   integer, intent(in), optional :: dl  ! ǥХå٥
-  integer :: i, nz, iz, iept
+  integer :: i, nz, iz, iept, iundef
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
   real :: undef
 
   nz=size(z)
   iept=0
+  iundef=0
 
   if(present(undeff))then
      undef=undeff
@@ -850,13 +875,13 @@ real function T_LFC( z_ref, z, temp, pres, qv, undeff, dl )  ! LFC ٤Ǥβ٤׻.
   end do
 
 !-- z_ref Ǥ̤׻.
-  call interpo_search_1d( z, z_ref, iz, int(undef) )
+  call interpo_search_1d( z, z_ref, iz, iundef )
   if(iz>=nz)then
-     iz=int(undef)
+     iz=iundef
   end if
 
 !-- iz  iz+1 Ǥ̤׻.
-  if(iz/=int(undef))then
+  if(iz/=iundef)then
   if(temp(iz)/=undef.and.qv(iz)/=undef.and.pres(iz)/=undef  &
   &  .and.temp(iz+1)/=undef.and.qv(iz+1)/=undef.and.pres(iz+1)/=undef)then
      eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
@@ -923,13 +948,14 @@ real function T_LNB( z_ref, z, temp, pres, qv, opt, undeff, dl )  ! LNB ٤Ǥβ٤
   integer, intent(in), optional :: opt  ! ׻ˡΥץ
   real, intent(in), optional :: undeff
   integer, intent(in), optional :: dl  ! ǥХå٥
-  integer :: i, nz, iz, iept, counter
+  integer :: i, nz, iz, iept, counter, iundef
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
   real :: undef
 
   nz=size(z)
   iept=0
+  iundef=0
 
   if(present(undeff))then
      undef=undeff
@@ -943,13 +969,13 @@ real function T_LNB( z_ref, z, temp, pres, qv, opt, undeff, dl )  ! LNB ٤Ǥβ٤
   end do
 
 !-- z_ref Ǥ̤׻.
-  call interpo_search_1d( z, z_ref, iz, int(undef) )
+  call interpo_search_1d( z, z_ref, iz, iundef )
   if(iz>=nz)then
-     iz=int(undef)
+     iz=iundef
   end if
 
 !-- iz  iz+1 Ǥ̤׻.
-  if(iz/=int(undef))then
+  if(iz/=iundef)then
   if(temp(iz)/=undef.and.qv(iz)/=undef.and.pres(iz)/=undef  &
   &  .and.temp(iz+1)/=undef.and.qv(iz+1)/=undef.and.pres(iz+1)/=undef)then
      eptiz=thetae_Bolton( temp(iz), qv(iz), pres(iz) )
