@@ -1,5 +1,5 @@
 !-----------------------------------------------------------------------
-!     Copyright (C) 2000-2013 GFD Dennou Club. All rights reserved.
+!     Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !-----------------------------------------------------------------------
 
 module Thermo_Advanced_Routine
@@ -781,7 +781,7 @@ end subroutine
 !--------------------------------------------------------
 !--------------------------------------------------------
 
-subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
+subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz, cord )
 ! ƥΥݥƥ󥷥뱲٤׻
   use Thermo_Function
   use Thermo_Routine
@@ -793,7 +793,7 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
   real, intent(in) :: u(size(x),size(y),size(z))  ! ®پ x ʬ [m/s]
   real, intent(in) :: v(size(x),size(y),size(z))  ! ®پ y ʬ [m/s]
   real, intent(in) :: w(size(x),size(y),size(z))  ! ®پ z ʬ [m/s]
-  real, intent(in) :: rho(size(x),size(y),size(z))  ! ̩پ [kg/m^3]
+  real, intent(in) :: rho(size(z))  ! ̩پ [kg/m^3]
   real, intent(in) :: pt(size(x),size(y),size(z))  ! ̾ [K]
   real, intent(in) :: cor(size(x),size(y))  ! ꥪѥ᡼ [/s]
   real, intent(inout) :: pv(size(x),size(y),size(z))  ! PV [Km^2/kgs]
@@ -801,6 +801,9 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
   real, intent(in), optional :: sx(size(x),size(y),size(z))  ! 󥰥ե
   real, intent(in), optional :: sy(size(x),size(y),size(z))  ! 󥰥ե
   real, intent(in), optional :: sz(size(x),size(y),size(z))  ! 󥰥ե
+  character(1), intent(in), optional :: cord   ! ľɸ
+                              ! 'z', 'p' =  [m],  [Pa]
+                              ! ǥե = 'z'
   real :: tmp1(size(x),size(y),size(z))
   real :: tmp2(size(x),size(y),size(z))
   real :: tmp3(size(x),size(y),size(z))
@@ -815,18 +818,17 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
   real :: scalex(size(x),size(y),size(z))
   real :: scaley(size(x),size(y),size(z))
   real :: scalez(size(x),size(y),size(z))
+  character(1) :: zp_flag
 
   nx=size(x)
   ny=size(y)
   nz=size(z)
 
-  if(present(sx).and.present(sy).and.present(sz))then
+  if(present(sx))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=sx(i,j,k)
-              scaley(i,j,k)=sy(i,j,k)
-              scalez(i,j,k)=sz(i,j,k)
            end do
         end do
      end do
@@ -835,13 +837,53 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(sy))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scaley(i,j,k)=sy(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scaley(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(sz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalez(i,j,k)=sz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scalez(i,j,k)=1.0
            end do
         end do
      end do
   end if
 
+  if(present(cord))then
+     zp_flag=cord
+  else
+     zp_flag='z'
+  end if
+
   if(present(undeff))then
 !  ̤ζָۤ׻.
      call grad_3d( x, y, z, pt, tmp1, tmp2, tmp3, undeff,  &
@@ -869,10 +911,11 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
      do k=1,nz
         do j=1,ny
            do i=1,nx
-              if(tmp7(i,j,k)==undeff.or.rho(i,j,k)==undeff)then
+              if(tmp3(i,j,k)==undeff.or.tmp7(i,j,k)==undeff.or.  &
+  &              rho(k)==undeff)then
                  pv(i,j,k)=undeff
               else
-                 pv(i,j,k)=(tmp7(i,j,k)+cor(i,j)*tmp3(i,j,k))/rho(i,j,k)
+                 pv(i,j,k)=(tmp7(i,j,k)+cor(i,j)*tmp3(i,j,k))/rho(k)
                  ! ׻Τб٤ʤΤ, ꥪ­碌.
               end if
            end do
@@ -890,7 +933,7 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
      do k=1,nz
         do j=1,ny
            do i=1,nx
-              pv(i,j,k)=(tmp7(i,j,k)+cor(i,j)*tmp3(i,j,k))/rho(i,j,k)
+              pv(i,j,k)=(tmp7(i,j,k)+cor(i,j)*tmp3(i,j,k))/rho(k)
            end do
         end do
      end do
@@ -902,6 +945,391 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
 
 end subroutine
 
+!--------------------------------------------------------
+!--------------------------------------------------------
+
+subroutine HEPV( x, y, z, u, v, rhoc, pt, cor, pv, undeff, sx, sy )
+! ϳضΩķϤǤ, ǤդαľɸϤˤ
+! ƥΥݥƥ󥷥뱲٤׻.
+  use Thermo_Function
+  use Thermo_Routine
+  use derivation
+  implicit none
+  real, intent(in) :: x(:)  ! x κɸѿ [MKS unit]
+  real, intent(in) :: y(:)  ! y κɸѿ [MKS unit]
+  real, intent(in) :: z(:)  ! z κɸѿ [MKS unit]
+  real, intent(in) :: u(size(x),size(y),size(z))  ! ®پ x ʬ [m/s]
+  real, intent(in) :: v(size(x),size(y),size(z))  ! ®پ y ʬ [m/s]
+  real, intent(in) :: rhoc(size(x),size(y),size(z))  !Ťդ̩ [MKS unit]
+  real, intent(in) :: pt(size(x),size(y),size(z))  ! 顼¸
+  real, intent(in) :: cor(size(x),size(y))  ! ꥪѥ᡼ [/s]
+  real, intent(inout) :: pv(size(x),size(y),size(z))  ! PV [Km^2/kgs]
+  real, intent(in), optional :: undeff
+  real, intent(in), optional :: sx(size(x),size(y))  ! 󥰥ե
+  real, intent(in), optional :: sy(size(x),size(y))  ! 󥰥ե
+  real :: tmp1(size(x),size(y),size(z))
+  real :: tmp2(size(x),size(y),size(z))
+  real :: tmp3(size(x),size(y),size(z))
+  real :: tmp4(size(x),size(y),size(z))
+  real :: tmp5(size(x),size(y),size(z))
+  real :: tmp6(size(x),size(y),size(z))
+  real :: tmp7(size(x),size(y),size(z))
+  integer :: i, j, k
+  integer :: nx  !  1 ǿ
+  integer :: ny  !  2 ǿ
+  integer :: nz  !  3 ǿ
+  real :: scalex(size(x),size(y))
+  real :: scaley(size(x),size(y))
+
+  nx=size(x)
+  ny=size(y)
+  nz=size(z)
+
+  if(present(sx))then
+     do j=1,ny
+        do i=1,nx
+           scalex(i,j)=sx(i,j)
+        end do
+     end do
+  else
+     do j=1,ny
+        do i=1,nx
+           scalex(i,j)=1.0
+        end do
+     end do
+  end if
+
+  if(present(sy))then
+     do j=1,ny
+        do i=1,nx
+           scaley(i,j)=sy(i,j)
+        end do
+     end do
+  else
+     do j=1,ny
+        do i=1,nx
+           scaley(i,j)=1.0
+        end do
+     end do
+  end if
+
+  if(present(undeff))then
+  !  顼¸̤ζָ, rotation ׻.
+!$omp parallel default(shared)
+!$omp do schedule(runtime) private(k)
+     do k=1,nz
+        call grad_2d( x, y, pt(:,:,k), tmp1(:,:,k), tmp2(:,:,k), undeff,  &
+  &                   hx=scalex, hy=scaley )
+        call curl( x, y, u(:,:,k), v(:,:,k), tmp6(:,:,k),  &
+  &                undeff, hx=scalex, hy=scaley )
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,j)
+     do j=1,ny
+        do i=1,nx
+           call grad_1d( z, pt(i,j,:), tmp3(i,j,:), undeff )
+           call grad_1d( z, v(i,j,:), tmp4(i,j,:), undeff )
+           call grad_1d( z, u(i,j,:), tmp5(i,j,:), undeff )
+        end do
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,j,k)
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              if(tmp4(i,j,k)/=undeff)then
+                 tmp4(i,j,k)=-tmp4(i,j,k)
+              end if
+              if(tmp6(i,j,k)/=undeff)then
+                 tmp6(i,j,k)=tmp6(i,j,k)+cor(i,j)
+              end if
+           end do
+        end do
+     end do
+!$omp end do
+!$omp end parallel
+
+  !  omega  grad pt Ѥ׻
+     call dot_prod_3d( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7, undeff )
+
+!$omp parallel default(shared)
+!$omp do schedule(dynamic) private(i,j,k)
+
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              if(tmp4(i,j,k)==undeff.or.rhoc(i,j,k)==undeff)then
+                 pv(i,j,k)=undeff
+              else
+                 pv(i,j,k)=tmp7(i,j,k)*rhoc(i,j,k)
+              end if
+           end do
+        end do
+     end do
+
+!$omp end do
+!$omp end parallel
+
+  else
+
+  !  顼¸̤ζָ, rotation ׻.
+!$omp parallel default(shared)
+!$omp do schedule(runtime) private(k)
+     do k=1,nz
+        call grad_2d( x, y, pt(:,:,k), tmp1(:,:,k), tmp2(:,:,k),  &
+  &                   hx=scalex, hy=scaley )
+        call curl( x, y, u(:,:,k), v(:,:,k), tmp6(:,:,k),  &
+  &                undeff, hx=scalex, hy=scaley )
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,j)
+     do j=1,ny
+        do i=1,nx
+           call grad_1d( z, pt(i,j,:), tmp3(i,j,:) )
+           call grad_1d( z, v(i,j,:), tmp4(i,j,:) )
+           call grad_1d( z, u(i,j,:), tmp5(i,j,:) )
+        end do
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,j,k)
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              tmp4(i,j,k)=-tmp4(i,j,k)
+              tmp6(i,j,k)=tmp6(i,j,k)+cor(i,j)
+           end do
+        end do
+     end do
+!$omp end do
+!$omp end parallel
+
+  !  omega  grad pt Ѥ׻
+     call dot_prod_3d( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7 )
+
+!$omp parallel default(shared)
+!$omp do schedule(runtime) private(i,j,k)
+
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              pv(i,j,k)=tmp7(i,j,k)*rhoc(i,j,k)
+           end do
+        end do
+     end do
+
+!$omp end do
+!$omp end parallel
+
+  end if
+
+end subroutine
+
+!--------------------------------------------------------
+!--------------------------------------------------------
+
+subroutine HQGPV( x, y, z, phi, t_ref, cor, qgpv, undef, hx, hy, rhoc )
+  ! Yoshino et al. (2003) : modified for (2).
+  use Thermo_Const
+  use Phys_Const
+  use Math_Const
+  use Thermo_Function
+  use Derivation
+
+  implicit none
+
+  real, intent(in) :: x(:)  ! x κɸѿ [MKS unit]
+  real, intent(in) :: y(:)  ! y κɸѿ [MKS unit]
+  real, intent(in) :: z(:)  ! z κɸѿ [MKS unit] (Z or P)
+  real, intent(in) :: t_ref(size(z))   ! (potential) temperature for basic [K]
+                      ! p-coord : temp, z-coord : potential temp.
+  real, intent(in) :: cor(size(x),size(y))  ! coriolis parameter [s-1]
+  real, intent(in) :: phi(size(x),size(y),size(z))
+                      ! p-coord : geopotential [J/kg], z-coord : pressure [Pa].
+  real, intent(inout) :: qgpv(size(x),size(y),size(z))   ! QGPV [s-1]
+  real, intent(in), optional :: undef
+  real, intent(in), optional :: hx(size(x),size(y))  ! 󥰥ե
+  real, intent(in), optional :: hy(size(x),size(y))  ! 󥰥ե
+  real, intent(in), optional :: rhoc(size(z))  ! basic den. for z-coord. [kg/m3]
+
+  integer :: nx, ny, nz, i, j, k
+  real :: kp, p_inv, fc, undeff
+  real, dimension(size(x),size(y)) :: sx, sy
+  real, dimension(size(x),size(y),size(z)) :: tmp
+  real, dimension(size(z)) :: sigma, dsdp, dtdp, d2tdp2, tmpz
+  logical :: z_flag
+
+  nx=size(x)
+  ny=size(y)
+  nz=size(z)
+
+  kp=Rd/Cpd
+
+  if(present(rhoc))then
+     z_flag=.true.
+  else
+     z_flag=.false.
+  end if
+
+  if(present(undef))then
+     undeff=undef
+  else
+     undeff=-999.0
+  end if
+
+  if(present(hx).and.present(hy))then
+     sx=hx
+     sy=hy
+  else
+     sx=1.0
+     sy=1.0
+  end if
+
+  fc=cor(nx/2,ny/2)
+
+  !-- calculating vertical stability
+  call grad_1d( z, t_ref, dtdp )
+  call laplacian_1d( z, t_ref, d2tdp2 )
+
+  if(z_flag.eqv..true.)then  ! -> Z-coord.
+
+!$omp parallel default(shared)
+!$omp do schedule(runtime) private(k)
+     do k=1,nz
+        sigma(k)=g*dtdp(k)/t_ref(k)
+        tmpz(k)=1.0/(rhoc(k)*sigma(k))
+     end do
+!$omp end do
+!$omp end parallel
+
+     call grad_1d( z, tmpz, dsdp )
+
+  else  ! -> P-coord.
+
+!$omp parallel default(shared)
+!$omp do schedule(runtime) private(k)
+     do k=1,nz
+        sigma(k)=(Rd/z(k))*(kp*t_ref(k)/z(k)-dtdp(k))
+        dsdp(k)=(-sigma(k)+Rd*(kp*(dtdp(k)-t_ref(k)/z(k))/z(k)-d2tdp2(k)))/z(k)
+     end do
+!$omp end do
+!$omp end parallel
+
+  end if
+
+  !-- calculating derivation for longitude
+
+!$omp parallel default(shared)
+!$omp do schedule(runtime) private(j)
+
+  do k=1,nz
+     call laplacian_2d( x, y, phi(:,:,k), tmp(:,:,k), undef=undeff,  &
+  &                     hx=sx, hy=sy )
+  end do
+
+!$omp end do
+
+!$omp barrier
+
+!$omp do schedule(runtime) private(i,j,k)
+
+  do k=1,nz
+     do j=1,ny
+        do i=1,nx
+           if(z_flag.eqv..true.)then
+              qgpv(i,j,k)=tmp(i,j,k)/(fc*rhoc(k))
+           else
+              qgpv(i,j,k)=tmp(i,j,k)/(fc)
+           end if
+        end do
+     end do
+  end do
+
+!$omp end do
+
+  !-- calculating derivation for altitude
+
+!$omp barrier
+
+!$omp do schedule(runtime) private(i,j)
+
+  do j=1,ny
+     do i=1,nx
+        call laplacian_1d( z, phi(i,j,:), tmp(i,j,:) )
+     end do
+  end do
+
+!$omp end do
+
+!$omp barrier
+
+!$omp do schedule(runtime) private(i,j,k)
+
+  do j=1,ny
+     do i=1,nx
+        do k=1,nz
+           if(z_flag.eqv..true.)then
+              qgpv(i,j,k)=qgpv(i,j,k)+tmp(i,j,k)*fc/(rhoc(k)*sigma(k))
+           else
+              qgpv(i,j,k)=qgpv(i,j,k)+tmp(i,j,k)*fc/sigma(k)
+           end if
+        end do
+     end do
+  end do
+
+!$omp end do
+
+!$omp barrier
+
+!$omp do schedule(runtime) private(i,j)
+
+  do j=1,ny
+     do i=1,nx
+        call grad_1d( z, phi(i,j,:), tmp(i,j,:) )
+     end do
+  end do
+
+!$omp end do
+
+!$omp barrier
+
+!$omp do schedule(runtime) private(i,j,k)
+
+  do j=1,ny
+     do i=1,nx
+        do k=1,nz
+           if(z_flag.eqv..true.)then
+              qgpv(i,j,k)=qgpv(i,j,k)+tmp(i,j,k)*fc*dsdp(k)
+           else
+              qgpv(i,j,k)=qgpv(i,j,k)-tmp(i,j,k)*fc*dsdp(k)/(sigma(k)*sigma(k))
+           end if
+        end do
+     end do
+  end do
+
+!$omp end do
+
+!$omp barrier
+
+!$omp do schedule(runtime) private(i,j,k)
+
+  do k=1,nz
+     do j=1,ny
+        do i=1,nx
+           qgpv(i,j,k)=qgpv(i,j,k)+cor(i,j)
+        end do     
+     end do     
+  end do
+
+!$omp end do
+!$omp end parallel
+
+end subroutine HQGPV
+
+
 !--------------------------------------------------------
 !--------------------------------------------------------
 !!!!!!!!!!!!!!!!!!!1 under construction
